import sys
import time
import psutil

MEGA = 2 ** 20
GIGA = 2 ** 30

def alloc_max_str(totaltobeleaked, memoryLeaksInSec, mbleakspertime, leaksINMB):
    '''
    Function to load memory by assigning string of requested size

    Arguments:
        memory: amount of memory to be utilized in MB
    Returns:
        a : String of size 'memory'
    '''
    i = 1
    a = ''

    li = []
    while True:

        #start_time = time.time()
        end_time = int((time.time()))+int(memoryLeaksInSec)
        try:
            a = ' ' * (int(i * int(((mbleakspertime))*MEGA)))
            li.append(a)
            if((psutil.virtual_memory().used >> 20) >= int(totaltobeleaked)):
                break
            elif sys.getsizeof(a) / MEGA == int(leaksINMB):
                break
            
        except MemoryError:
            break
        time.sleep(end_time-time.time())
        i += 1
    del li 
    return a

def pmem():
    '''
    Function to display memory statistics
    '''
    tot, avail, percent, used, free = psutil.virtual_memory()[0:5]
    tot, avail, used, free = tot / GIGA, avail / GIGA, used / GIGA, free / GIGA
    print("Memory Stats: total = %s GB \navail = %s GB \nused = %s GB \nfree = %s GB \npercent = %s"
          % (tot, avail, used, free, percent))


def memory_stress(totaltobeleaked, memoryLeaksInSec, totalDurationInSec, mbleakspertime, leaksINMB):
    '''
    Function to stress memory and display memory Stats

    Arguments:
        memory: amount of memory to be utilized in MB
        exec_time: time for which the system is supposed to keep the object

    Returns:
        a : String of size 'memory'
    '''
    pmem()
    obj = alloc_max_str(totaltobeleaked, memoryLeaksInSec, mbleakspertime, leaksINMB)
    pmem()
    return obj

def apply_memory_leak(leaksINMB,totalDurationInSec, memoryLeaksInSec):
    totaltobeleaked = int(int(psutil.virtual_memory().used >> 20) + int(leaksINMB))
    mbleakspertime = int(leaksINMB)/(int(totalDurationInSec)/int(memoryLeaksInSec))
    memory_stress(totaltobeleaked, memoryLeaksInSec, totalDurationInSec, mbleakspertime, leaksINMB)

#need to create a check for memory should not be greater than total from psutil.virtual_memory().total




import time
import sys
import random


def get_current_timestamp_in_ms():
    if sys.version_info >= (3, 7, 0):
        time_stamp = time.time_ns() // 1000000
    else:
        time_stamp = int(round(time.time() * 1000))

    return time_stamp


def generate_flow_path_id():
    # 63 bit number having 1 followed by 62 zeros
    # In 64 bit long long, 1 bit is for sign (+ or -)
    # So effective range is of 63 bits only
    # Flow Path Id (63 bits) Format:
    # DC Id (8 bits) App Id (10 bits) Timestamp ID (38 bits) Unique ID (7 bits)

    flow_path_id = 0x4000000000000000
    
    dc_id, dc_mask, dc_pos = 0, 0x0F, 56
    app_id, app_mask, app_pos = 0, 0x03FF, 46
    timestamp_id, timestamp_mask, timestamp_pos = get_current_timestamp_in_ms(), 0x3FFFFFFFFF, 8
    unique_id, unique_mask, unique_pos = random.getrandbits(7), 0xFF, 0

    dc_and = (dc_id & dc_mask) << dc_pos
    app_and = (app_id & app_mask) << app_pos
    timestamp_and = (timestamp_id & timestamp_mask) << timestamp_pos
    unique_and = (unique_id & unique_mask) << unique_pos

    flow_path_id = flow_path_id + dc_and + app_and + timestamp_and + unique_and
    return flow_path_id


def get_utf8_bytes(str_obj):
    try:
        bytes_obj = bytes(str_obj, 'utf-8')
    except:
        bytes_obj = bytes(str_obj.encode("utf-8"))

    return bytes_obj
